@@ -36,6 +36,8 @@
 #include "introspection.h"
 #include "object.h"
 
+#define MAX_CHILDREN 65536
+
 typedef struct _MatchRulePrivate MatchRulePrivate;
 struct _MatchRulePrivate
 {
@@ -355,7 +357,7 @@ match_attributes_all_p (AtkObject *child, AtkAttributeSet *attributes)
       AtkAttribute *attr = g_slist_nth_data (attributes, i);
       for (k = 0; k < oa_length; k++)
         {
-          AtkAttribute *oa_attr = g_slist_nth_data (attributes, i);
+          AtkAttribute *oa_attr = g_slist_nth_data (oa, k);
           if (!g_ascii_strcasecmp (oa_attr->name, attr->name) &&
               !g_ascii_strcasecmp (oa_attr->value, attr->value))
             {
@@ -428,7 +430,7 @@ match_attributes_none_p (AtkObject *child, AtkAttributeSet *attributes)
       AtkAttribute *attr = g_slist_nth_data (attributes, i);
       for (k = 0; k < oa_length; k++)
         {
-          AtkAttribute *oa_attr = g_slist_nth_data (attributes, i);
+          AtkAttribute *oa_attr = g_slist_nth_data (oa, k);
           if (!g_ascii_strcasecmp (oa_attr->name, attr->name) &&
               !g_ascii_strcasecmp (oa_attr->value, attr->value))
             {
@@ -484,6 +486,8 @@ sort_order_canonical (MatchRulePrivate *mrp, GList *ls, gint kount, gint max, At
   glong acount = atk_object_get_n_accessible_children (obj);
   gboolean prev = pobj ? TRUE : FALSE;
 
+  if (acount > MAX_CHILDREN)
+    acount = MAX_CHILDREN;
   for (; i < acount && (max == 0 || kount < max); i++)
     {
       AtkObject *child = atk_object_ref_accessible_child (obj, i);
@@ -551,9 +555,11 @@ sort_order_rev_canonical (MatchRulePrivate *mrp, GList *ls, gint kount, gint max
       while (nextobj && atk_object_get_n_accessible_children (nextobj) > 0)
         {
           AtkObject *follow;
+          gint count = atk_object_get_n_accessible_children (nextobj);
+          if (count > MAX_CHILDREN)
+            count = MAX_CHILDREN;
 
-          follow = atk_object_ref_accessible_child (nextobj,
-                                                    atk_object_get_n_accessible_children (nextobj) - 1);
+          follow = atk_object_ref_accessible_child (nextobj, count - 1);
           g_object_unref (nextobj);
           nextobj = follow;
         }
@@ -1123,6 +1129,8 @@ append_accessible_properties (DBusMessageIter *iter, AtkObject *obj, GArray *pro
         return;
     }
   count = atk_object_get_n_accessible_children (obj);
+  if (count > MAX_CHILDREN)
+    count = MAX_CHILDREN;
   for (i = 0; i < count; i++)
     {
       AtkObject *child = atk_object_ref_accessible_child (obj, i);
