@@ -2016,3 +2016,51 @@ _atspi_strdup_and_adjust_for_dbus (const char *s)
   d[0] = toupper (d[0]);
   return d;
 }
+
+/**
+ * atspi_get_version:
+ * @major: (out): the major version.
+ * @minor: (out): the minor version.
+ * @micro: (out): the micro/patch version.
+ *
+ * Returns the version of the AT-SPI library being used at runtime.
+
+* Since: 2.50
+ */
+void
+atspi_get_version (gint *major, gint *minor, gint *micro)
+{
+  if (major)
+    *major = ATSPI_MAJOR_VERSION;
+  if (minor)
+    *minor = ATSPI_MINOR_VERSION;
+  if (micro)
+    *micro = ATSPI_MICRO_VERSION;
+}
+
+gboolean
+_atspi_key_is_on_keypad (gint keycode)
+{
+  switch (keycode)
+    {
+    case 106: /* / */
+    case 63:  /* * */
+    case 82:  /* - */
+    case 79:  /* 7 */
+    case 80:  /* 8 */
+    case 91:  /* 9 */
+    case 86:  /* + */
+    case 83:  /* 4 */
+    case 84:  /* 5 */
+    case 85:  /* 6 */
+    case 87:  /* 1 */
+    case 88:  /* 2 */
+    case 89:  /* 3 */
+    case 104: /* enter */
+    case 90:  /* 0 */
+    case 81:  /* . */
+      return TRUE;
+    default:
+      return FALSE;
+    }
+}
